@@ -194,11 +194,17 @@ pub(crate) struct ComponentExtraData {
     pub(crate) globals: HashMap<String, Pin<Rc<dyn crate::global_component::GlobalComponent>>>,
     pub(crate) self_weak:
         once_cell::unsync::OnceCell<vtable::VWeak<ComponentVTable, ErasedComponentBox>>,
+    // resource id -> file path
+    pub(crate) embedded_file_resources: HashMap<usize, String>,
 }
 
 impl Default for ComponentExtraData {
     fn default() -> Self {
-        Self { globals: HashMap::new(), self_weak: Default::default() }
+        Self {
+            globals: HashMap::new(),
+            self_weak: Default::default(),
+            embedded_file_resources: Default::default(),
+        }
     }
 }
 
@@ -300,6 +306,27 @@ pub struct ComponentDescription<'id> {
 
     /// compiled globals
     compiled_globals: Vec<crate::global_component::CompiledGlobal>,
+    /// Map of all exported global singletons and their index in the compiled_globals vector. The key
+    /// is the normalized name of the global.
+    exported_globals_by_name: BTreeMap<String, usize>,
+}
+
+fn internal_properties_to_public<'a>(
+    prop_iter: impl Iterator<Item = (&'a String, &'a PropertyDeclaration)> + 'a,
+) -> impl Iterator<Item = (String, sixtyfps_compilerlib::langtype::Type)> + 'a {
+    prop_iter.filter(|(_, v)| v.expose_in_public_api).map(|(s, v)| {
+        let name = v
+            .node
+            .as_ref()
+            .and_then(|n| {
+                n.as_ref()
+                    .either(|n| n.DeclaredIdentifier(), |n| n.DeclaredIdentifier())
+                    .child_token(parser::SyntaxKind::Identifier)
+            })
+            .map(|n| n.to_string())
+            .unwrap_or_else(|| s.clone());
+        (name, v.property_type.clone())
+    })
 }
 
 impl<'id> ComponentDescription<'id> {
@@ -314,19 +341,25 @@ impl<'id> ComponentDescription<'id> {
     pub fn properties(
         &self,
     ) -> impl Iterator<Item = (String, sixtyfps_compilerlib::langtype::Type)> + '_ {
-        self.public_properties.iter().filter(|(_, v)| v.expose_in_public_api).map(|(s, v)| {
-            let name = v
-                .node
-                .as_ref()
-                .and_then(|n| {
-                    n.as_ref()
-                        .either(|n| n.DeclaredIdentifier(), |n| n.DeclaredIdentifier())
-                        .child_token(parser::SyntaxKind::Identifier)
-                })
-                .map(|n| n.to_string())
-                .unwrap_or_else(|| s.clone());
-            (name, v.property_type.clone())
-        })
+        internal_properties_to_public(self.public_properties.iter())
+    }
+
+    /// List names of exported global singletons
+    pub fn global_names(&self) -> impl Iterator<Item = String> + '_ {
+        self.compiled_globals
+            .iter()
+            .filter(|g| g.visible_in_public_api())
+            .flat_map(|g| g.names().into_iter())
+    }
+
+    pub fn global_properties(
+        &self,
+        name: &str,
+    ) -> Option<impl Iterator<Item = (String, sixtyfps_compilerlib::langtype::Type)> + '_> {
+        self.exported_globals_by_name
+            .get(crate::normalize_identifier(name).as_ref())
+            .and_then(|global_idx| self.compiled_globals.get(*global_idx))
+            .map(|global| internal_properties_to_public(global.public_properties()))
     }
 
     /// Instantiate a runtime component from this ComponentDescription
@@ -862,6 +895,14 @@ pub(crate) fn generate_component<'id>(
                 "TextOverflow" => property_info::<sixtyfps_corelib::items::TextOverflow>(),
                 "ImageFit" => property_info::<sixtyfps_corelib::items::ImageFit>(),
                 "FillRule" => property_info::<sixtyfps_corelib::items::FillRule>(),
+                "StandardButtonKind" => {
+                    property_info::<sixtyfps_corelib::items::StandardButtonKind>()
+                }
+                "DialogButtonRole" => property_info::<sixtyfps_corelib::items::DialogButtonRole>(),
+                "PointerEventButton" => {
+                    property_info::<sixtyfps_corelib::items::PointerEventButton>()
+                }
+                "PointerEventKind" => property_info::<sixtyfps_corelib::items::PointerEventKind>(),
                 _ => panic!("unknown enum"),
             },
             Type::LayoutCache => property_info::<SharedVector<f32>>(),
@@ -904,12 +945,33 @@ pub(crate) fn generate_component<'id>(
 
     let public_properties = component.root_element.borrow().property_declarations.clone();
 
+    let mut exported_globals_by_name: BTreeMap<String, usize> = Default::default();
+
     let compiled_globals = component
         .used_types
         .borrow()
         .globals
         .iter()
-        .map(crate::global_component::generate)
+        .enumerate()
+        .map(|(index, component)| {
+            let mut global = crate::global_component::generate(component);
+
+            if component.visible_in_public_api() {
+                global.extend_public_properties(
+                    component.root_element.borrow().property_declarations.clone().into_iter(),
+                );
+
+                exported_globals_by_name.extend(
+                    component
+                        .exported_global_names
+                        .borrow()
+                        .iter()
+                        .map(|exported_name| (exported_name.name.clone(), index)),
+                )
+            }
+
+            global
+        })
         .collect();
 
     let t = ComponentVTable {
@@ -935,6 +997,7 @@ pub(crate) fn generate_component<'id>(
         extra_data_offset,
         public_properties,
         compiled_globals,
+        exported_globals_by_name,
     };
 
     Rc::new(t)
@@ -1014,10 +1077,21 @@ pub fn instantiate(
             .iter()
             .map(|g| {
                 let (_, instance) = crate::global_component::instantiate(g);
-                g.names().iter().map(|name| (name.clone(), instance.clone())).collect::<Vec<_>>()
+                g.names()
+                    .iter()
+                    .map(|name| (crate::normalize_identifier(name).to_string(), instance.clone()))
+                    .collect::<Vec<_>>()
             })
             .flatten()
             .collect();
+
+        extra_data.embedded_file_resources = component_type
+            .original
+            .embedded_file_resources
+            .borrow()
+            .iter()
+            .map(|(path, id)| (*id, path.clone()))
+            .collect();
     }
     *component_type.window_offset.apply_mut(instance.as_mut()) =
         window.map(|window| window.clone().into());
@@ -1484,6 +1558,33 @@ impl<'a, 'id> InstanceRef<'a, 'id> {
     pub fn window(&self) -> &sixtyfps_corelib::window::api::Window {
         self.component_type.window_offset.apply(self.as_ref()).as_ref().as_ref().unwrap()
     }
+
+    pub fn parent_instance(&self) -> Option<InstanceRef<'a, 'id>> {
+        if let Some(parent_offset) = self.component_type.parent_component_offset {
+            if let Some(parent) = parent_offset.apply(self.as_ref()) {
+                let parent_instance = unsafe {
+                    Self {
+                        instance: Pin::new_unchecked(
+                            &*(parent.as_ref().as_ptr() as *const Instance<'id>),
+                        ),
+                        component_type: &*(Pin::into_inner_unchecked(*parent).get_vtable()
+                            as *const ComponentVTable
+                            as *const ComponentDescription<'id>),
+                    }
+                };
+                return Some(parent_instance);
+            };
+        }
+        None
+    }
+
+    pub fn toplevel_instance(&self) -> InstanceRef<'a, 'id> {
+        if let Some(parent) = self.parent_instance() {
+            parent.toplevel_instance()
+        } else {
+            *self
+        }
+    }
 }
 
 /// Show the popup at the given location
