@@ -469,7 +469,7 @@ pub fn eval_expression(expression: &Expression, local_context: &mut EvalLocalCon
                 (sub, op) => panic!("unsupported {} {:?}", op, sub),
             }
         }
-        Expression::ImageReference(resource_ref) => {
+        Expression::ImageReference{ resource_ref, .. } => {
             Value::Image(match resource_ref {
                 sixtyfps_compilerlib::expression_tree::ImageReference::None => {
                     Ok(Default::default())
@@ -477,7 +477,31 @@ pub fn eval_expression(expression: &Expression, local_context: &mut EvalLocalCon
                 sixtyfps_compilerlib::expression_tree::ImageReference::AbsolutePath(path) => {
                     corelib::graphics::Image::load_from_path(std::path::Path::new(path))
                 }
-                sixtyfps_compilerlib::expression_tree::ImageReference::EmbeddedData { .. } => panic!("Resource embedding is not supported by the interpreter")
+                sixtyfps_compilerlib::expression_tree::ImageReference::EmbeddedData { resource_id, extension } => {
+                    let toplevel_instance = match local_context.component_instance {
+                        ComponentInstance::InstanceRef(instance) => instance.toplevel_instance(),
+                        ComponentInstance::GlobalComponent(_) => unimplemented!(),
+                    };
+                    let extra_data = toplevel_instance.component_type.extra_data_offset.apply(toplevel_instance.as_ref());
+                    let path = extra_data.embedded_file_resources.get(resource_id).expect("internal error: invalid resource id");
+
+                    let virtual_file = sixtyfps_compilerlib::fileaccess::load_file(std::path::Path::new(path)).unwrap();  // embedding pass ensured that the file exists
+
+                    if let (std::borrow::Cow::Borrowed(static_path), Some(static_data)) = (virtual_file.path, virtual_file.builtin_contents) {
+                        let virtual_file_extension = std::path::Path::new(static_path).extension().unwrap().to_str().unwrap();
+                        debug_assert_eq!(virtual_file_extension, extension);
+                        Ok(corelib::graphics::Image::from(
+                            corelib::graphics::ImageInner::EmbeddedData {
+                                data: corelib::slice::Slice::from_slice(static_data),
+                                format: corelib::slice::Slice::from_slice(virtual_file_extension.as_bytes())
+                            }
+                        ))
+                    } else {
+                        corelib::debug_log!("Cannot embed images from disk {}", path);
+                        Ok(corelib::graphics::Image::default())
+
+                    }
+                }
             }.unwrap_or_else(|_| {
                 eprintln!("Could not load image {:?}",resource_ref );
                 Default::default()
@@ -713,6 +737,19 @@ pub fn store_property(
     let component = element.borrow().enclosing_component.upgrade().unwrap();
     if element.borrow().id == component.root_element.borrow().id {
         if let Some(x) = enclosing_component.component_type.custom_properties.get(name) {
+            if let Some(orig_decl) = enclosing_component
+                .component_type
+                .original
+                .root_element
+                .borrow()
+                .property_declarations
+                .get(name)
+            {
+                // Do an extra type checking because PropertyInfo::set won't do it for custom structures or array
+                if !check_value_type(&value, &orig_decl.property_type) {
+                    return Err(SetPropertyError::WrongType);
+                }
+            }
             unsafe {
                 let p = Pin::new_unchecked(&*enclosing_component.as_ptr().add(x.offset));
                 return x
@@ -727,10 +764,54 @@ pub fn store_property(
     let item_info = &enclosing_component.component_type.items[element.borrow().id.as_str()];
     let item = unsafe { item_info.item_from_component(enclosing_component.as_ptr()) };
     let p = &item_info.rtti.properties.get(name).ok_or(SetPropertyError::NoSuchProperty)?;
-    p.set(item, value, maybe_animation.as_animation()).unwrap();
+    p.set(item, value, maybe_animation.as_animation()).map_err(|()| SetPropertyError::WrongType)?;
     Ok(())
 }
 
+fn check_value_type(value: &Value, ty: &Type) -> bool {
+    match ty {
+        Type::Void => true,
+        Type::Invalid
+        | Type::InferredProperty
+        | Type::InferredCallback
+        | Type::Component(_)
+        | Type::Builtin(_)
+        | Type::Native(_)
+        | Type::Callback { .. }
+        | Type::Function { .. }
+        | Type::ElementReference => panic!("not valid property type"),
+        Type::Float32 => matches!(value, Value::Number(_)),
+        Type::Int32 => matches!(value, Value::Number(_)),
+        Type::String => matches!(value, Value::String(_)),
+        Type::Color => matches!(value, Value::Brush(_)),
+        Type::UnitProduct(_)
+        | Type::Duration
+        | Type::PhysicalLength
+        | Type::LogicalLength
+        | Type::Angle
+        | Type::Percent => matches!(value, Value::Number(_)),
+        Type::Image => matches!(value, Value::Image(_)),
+        Type::Bool => matches!(value, Value::Bool(_)),
+        Type::Model => {
+            matches!(value, Value::Model(_) | Value::Bool(_) | Value::Number(_) | Value::Array(_))
+        }
+        Type::PathElements => matches!(value, Value::PathElements(_)),
+        Type::Easing => matches!(value, Value::EasingCurve(_)),
+        Type::Brush => matches!(value, Value::Brush(_)),
+        Type::Array(inner) => {
+            matches!(value, Value::Model(_))
+                || matches!(value, Value::Array(vec) if vec.iter().all(|x| check_value_type(x, inner)))
+        }
+        Type::Struct { fields, .. } => {
+            matches!(value, Value::Struct(str) if str.iter().all(|(k, v)| fields.get(k).map_or(false, |ty| check_value_type(v, ty))))
+        }
+        Type::Enumeration(en) => {
+            matches!(value, Value::EnumerationValue(name, _) if name == en.name.as_str())
+        }
+        Type::LayoutCache => matches!(value, Value::LayoutCache(_)),
+    }
+}
+
 pub(crate) fn invoke_callback(
     component_instance: ComponentInstance,
     element: &ElementRc,
